@@ -25,48 +25,68 @@
 #ifndef HANDLERS_H
 #define HANDLERS_H
 
+#include <stdbool.h>
+
+#include "scanmem.h"
+
+/* Common documentations */
+
+#define SET_FORMAT_DOC \
+    "The Set Format:\n" \
+    "[!][..a](,b..c | d, ...)[e..]\n" \
+    "An optional `!` at the beginning inverts the defined set.\n" \
+    "The beginning and end of the set may include an optional \"open-ended\" range:\n" \
+    "`..a` means a range starting at `0` to `a`.\n" \
+    "`e..` means a range starting at `e` to the last valid value.\n" \
+    "The rest is the same as the standard format.\n\n"
+
 /*
  * SHRTDOC's are one line descriptions of the supported command (shown with `help`).
  * LONGDOC's are detailed descriptions (shown with `help command`) (wrap them before column 80).
  * 
  * The DOC's are passed to the registercommand() routine, and are read by the help
- * command. You can define SHRTDOC to NULL and help will not print it.
- * However, a NULL longdoc will print "missing documentation" if specific help is requested,
- * if you really dont want it to print anything, use "".
+ * command. You can define SHRTDOC as NULL and help will not print it.
+ * However, a NULL longdoc will print "missing documentation" if specific help is requested.
+ * If you really don't want it to print anything, use "".
  *
- * Each handler is passed an argv and argc, argv[0] is the command called, along with
+ * Each handler is passed an argv and argc, argv[0] being the command called. Also passed is
  * a pointer to the global settings structure, which contains various settings and lists
  * that handlers are allowed to change. One handler can handle multiple commands, but
  * you need register each command with its own documentation.
  */
 
 #define SET_SHRTDOC "change known matches to specified value"
-#define SET_LONGDOC "usage: set [match-id[,match-id,...]=]n[/d] [...]\n" \
-					"Inject the value `n` into the match numbers `match-id`, or if just `n` is\n" \
+#define SET_LONGDOC "usage: set <[match-id set=]n [...]>\n" \
+               "Inject the value `n` into all the matches in `match-id set`, or if just `n` is\n" \
                "specified, all known matches. `n` can be specified in standard C language\n" \
                "notation, a leading 0x for hexadecimal, leading 0 for octal, and everything\n" \
                "else is assumed to be decimal. All known matches, along with their match-ids\n" \
                "can be displayed with the `list` command. Multiple match-ids can be specified,\n" \
-               "separated with commas and terminated with an '=' sign.\n" \
+               "as a set data-type, terminated with a '=' sign. See below for a description of\n" \
+               "the set data-type.\n" \
                "To set a value continually, for example to prevent a counter from decreasing,\n" \
                "suffix the command with '/', followed by the number of seconds to wait between\n" \
                "sets. Interrupt scanmem with ^C to stop the setting.\n\n" \
                "Note that this command cannot work for bytearray or string.\n\n" \
+               SET_FORMAT_DOC \
                "Examples:\n" \
                "\tset 10 - set all known matches to 10\n" \
                "\tset 0=0x03 - set match 0 to 0x03.\n" \
-               "\tset 0,1,2,7=0x32 - set matches 0,1,2 and 7 to 0x32\n" \
+               "\tset ..7=0x32 - set matches 0 through 7 to 0x32\n" \
                "\tset 0,3=42/8 - set matches 0 and 3 to 42 every 8 seconds\n" \
-               "\tset 12,13,14=0x23/2 23,19=0/8 6=4 0 - complex example, can be combined" \
+               "\tset !4..12=5 - set all matches except 4 through 12 to 5\n" \
+               "\tset 12,13,14=0x23/2 19..23=0/8 6=4 0 - complex example, can be combined" \
 
 bool handler__set(globals_t *vars, char **argv, unsigned argc);
 
-#define LIST_SHRTDOC "list all currently known matches"
-#define LIST_LONGDOC "usage: list\n" \
-               "Print all the currently known matches, along with details about the\n" \
+#define LIST_SHRTDOC "list currently known matches"
+#define LIST_LONGDOC "usage: list [max_to_print]\n" \
+               "Print currently known matches, along with details about the\n" \
                "match, such as its type, location, and last known value. The number in\n" \
                "the left column is the `match-id`, this can be passed to other commands\n" \
                "such as `set`, `delete`, etc.\n" \
+               "By default `list` prints up to 10k matches, a numerical parameter" \
+               "can be given to change this limit.\n" \
                "The flags displayed indicate the possible types of the variable.\n" \
                "Also the region id, an offset and the region type belonging to a match\n" \
                "are displayed. The offset is used from the code load address or region start.\n" \
@@ -74,16 +94,22 @@ bool handler__set(globals_t *vars, char **argv, unsigned argc);
 
 bool handler__list(globals_t *vars, char **argv, unsigned argc);
 
-#define DELETE_SHRTDOC "delete a known match by match-id"
-#define DELETE_LONGDOC "usage: delete match-id\n" \
-                "Remove the match `match-id` from the match list. The `match-id` can\n" \
+#define DELETE_SHRTDOC "delete known matches by match-id"
+#define DELETE_LONGDOC "usage: delete <match-id set>\n" \
+                "Remove a set of match-id's from the match list. The set format\n" \
+                "is described below. The list of match-id's can\n" \
                 "be found using the `list` command. To delete all matches, see\n" \
                 "the `reset` command.\n" \
                 "To delete all matches associated with a particular library, see the\n" \
-                "`dregion` command, which will also remove any associated matches.\n" \
-                "Example:\n" \
-                "\tdelete 0 - delete match 0\n" \
-                "NOTE: match-ids may be recalculated after matches are removed or added."
+                "`dregion` command, which will also remove any associated matches.\n\n" \
+                SET_FORMAT_DOC \
+                "Examples:\n" \
+                "\tdelete ..5,10..15  - delete matches 0 through 5 and 10 through 15\n" \
+                "\tdelete ..5,14,20.. - delete matches 0 through 5, 14, and 20 through the last match\n" \
+                "\tdelete !4..8,11    - delete all matches except for 4 through 8 and 11\n\n" \
+                "NOTE: Match-ids may be recalculated after matches are removed or added. However, the set\n" \
+                "      of matches is guaranteed to be deleted properly."
+
 
 bool handler__delete(globals_t *vars, char **argv, unsigned argc);
 
@@ -114,10 +140,11 @@ bool handler__pid(globals_t *vars, char **argv, unsigned argc);
 bool handler__snapshot(globals_t *vars, char **argv, unsigned argc);
 
 #define DREGION_SHRTDOC "delete a known region by region-id"
-#define DREGION_LONGDOC "usage: dregion [!]region-id[,region-id[,...]]\n" \
-                "Remove the region `region-id` from the regions list, along with any matches\n" \
-                "affected from the match list. The `region-id` can be found using the `lregions`\n" \
-                "command. A leading `!` indicates the list should be inverted.\n" 
+#define DREGION_LONGDOC "usage: dregion <region-id set>\n" \
+                "Remove a set of region-id from the regions list,\n" \
+                "along with any matches affected from the match list.\n" \
+                "The `region-id` can be found using the `lregions` command.\n\n" \
+                SET_FORMAT_DOC
 
 bool handler__dregion(globals_t *vars, char **argv, unsigned argc);
 
@@ -168,7 +195,7 @@ bool handler__lregions(globals_t *vars, char **argv, unsigned argc);
                 "You can use this in conjunction with `snapshot` if you never know its value."
 
 
-bool handler__decinc(globals_t *vars, char **argv, unsigned argc);
+bool handler__operators(globals_t *vars, char **argv, unsigned argc);
 
 #define VERSION_SHRTDOC "print current version"
 #define VERSION_LONGDOC "usage: version\n" \
@@ -229,11 +256,11 @@ bool handler__update(globals_t *vars, char **argv, unsigned argc);
 bool handler__eof(globals_t *vars, char **argv, unsigned argc);
 
 #define SHELL_SHRTDOC "execute a shell command without leaving scanmem"
-#define SHELL_LONGDOC "usage: shell [shell-command]\n" \
+#define SHELL_LONGDOC "usage: ! | shell [shell-command]\n" \
                 "Execute `shell-command` using /bin/sh then return, useful for reading man\n" \
                 "pages, checking top, or making notes with an external editor.\n" \
                 "Examples:\n" \
-                "\tshell vi notes.txt\n" \
+                "\t! vi notes.txt\n" \
                 "\tshell man scanmem\n" \
                 "\tshell cat > notes.txt\n"
 
@@ -273,11 +300,13 @@ bool handler__dump(globals_t *vars, char **argv, unsigned argc);
                 "\tint{8|16|32|64} (or i{8|16|32|64} for short)\n" \
                 "\tfloat{32|64} (or f{32|64} for short)\n" \
                 "\tbytearray\n" \
+                "\tstring\n" \
                 "\n" \
                 "Example:\n" \
                 "\twrite i16 60103e 0\n" \
                 "\twrite float32 60103e 0\n" \
-                "\twrite bytearray 60103e ff 01 32\n"
+                "\twrite bytearray 60103e ff 01 32\n" \
+                "\twrite string 60103e cheating\n"
 
 bool handler__write(globals_t *vars, char **argv, unsigned argc);
 
@@ -307,14 +336,6 @@ bool handler__write(globals_t *vars, char **argv, unsigned argc);
                  "\t2:\theap, stack executable and bss only\n" \
                  "\t3:\teverything(e.g. other libs)\n" \
                  "\n" \
-                 "detect_reverse_change\twhether to (also) search for values that changes oppositely as given order\n" \
-                 "\t\t\tDefault:0\n" \
-                 "\tIf you want to use this feature, you can only search for INCREASED or DECREASED after initial search\n" \
-                 "\n" \
-                 "\tpossible values:\n" \
-                 "\t0:\tdisabled\n" \
-                 "\t1:\tenabled\n" \
-                 "\n" \
                  "dump_with_ascii\twhether to print ascii characters with a memory dump\n" \
                  "\t\t\tDefault:1\n" \
                  "\n" \
