@@ -1,11 +1,24 @@
 /*
+    Just some license texts.
 
- $Id: licence.h,v 1.2 2007-04-11 10:43:26+01 taviso Exp $
+    This file is part of libscanmem.
 
+    This library is free software: you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as published
+    by the Free Software Foundation; either version 3 of the License, or
+    (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public License
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#ifndef _INC_LICENCE
-#define _INC_LICENCE 
+#ifndef LICENCE_H
+#define LICENCE_H
 
 #define SM_WARRANTY \
         "15. Disclaimer of Warranty.\n\n" \
@@ -44,4 +57,4 @@
         "You should have received a copy of the GNU General Public License\n" \
         "along with this program; if not, see <http://www.gnu.org/licenses/>.\n" 
 
-#endif 
+#endif /* LICENCE_H */
