@@ -1,23 +1,23 @@
 /*
-    Prompt, command completion and version information.
+    Prompt and command completion.
 
     Copyright (C) 2006,2007,2009 Tavis Ormandy <taviso@sdf.lonestar.org>
     Copyright (C) 2010,2011 Lu Wang <coolwanglu@gmail.com>
 
-    This file is part of libscanmem.
+    This file is part of scanmem.
 
-    This library is free software: you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as published
-    by the Free Software Foundation; either version 3 of the License, or
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
+    This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
+    GNU General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public License
-    along with this library.  If not, see <http://www.gnu.org/licenses/>.
+    You should have received a copy of the GNU General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef _GNU_SOURCE
@@ -40,6 +40,7 @@
 #include "readline.h"
 #endif
 
+#include "menu.h"
 #include "getline.h"
 #include "scanmem.h"
 #include "commands.h"
@@ -102,46 +103,34 @@ static char **commandcompletion(const char *text, int start, int end)
 
 
 /*
- * sm_getcommand() reads in a command using readline, and places a pointer to
- * the read string into *line, _which must be free'd by caller_.
+ * getcommand() reads in a command using readline and places a pointer to
+ * the read string into *line, which must be free'd by caller.
  * returns true on success, or false on error.
  */
 
-bool sm_getcommand(globals_t *vars, char **line)
+bool getcommand(globals_t *vars, char **line)
 {
     char prompt[64];
     bool success = true;
 
     assert(vars != NULL);
 
-    snprintf(prompt, sizeof(prompt), "%ld> ", vars->num_matches);
+    if (vars->matches) {
+        snprintf(prompt, sizeof(prompt), "%ld> ", vars->num_matches);
+    } else {
+        snprintf(prompt, sizeof(prompt), "> ");
+    }
 
     rl_readline_name = "scanmem";
     rl_attempted_completion_function = commandcompletion;
 
     while (true) {
-        if (vars->options.backend == 0)
-        {
-            /* for normal user, read in the next command using readline library */
-            success = ((*line = readline(prompt)) != NULL);
-        }
-        else 
-        {
-            /* disable readline for front-end, since readline may produce ansi escape codes, which is terrible for front-end */
-            printf("%s\n", prompt); /* add a newline for front-end */
-            fflush(stdout); /* otherwise front-end may not receive this */
-            *line = NULL; /* let getline malloc it */
-            size_t n;
-            ssize_t bytes_read = getline(line, &n, stdin);
-            success = (bytes_read > 0);
-            if (success)
-                (*line)[bytes_read-1] = '\0'; /* remove the trialing newline */
-        }
+
+        success = ((*line = readline(prompt)) != NULL);
         if (!success) {
             /* EOF */
             if ((*line = strdup("__eof")) == NULL) {
-                fprintf(stderr,
-                        "error: sorry, there was a memory allocation error.\n");
+                show_error("sorry, there was a memory allocation error.\n");
                 return false;
             }
         }
