@@ -5,6 +5,7 @@
     Copyright (C) 2009           Eli Dupree <elidupree@charter.net>
     Copyright (C) 2009,2010      WANG Lu <coolwanglu@gmail.com>
     Copyright (C) 2015           Sebastian Parschauer <s.parschauer@gmx.de>
+    Copyright (C) 2017           Andrea Stacchiotti <andreastacchiotti(a)gmail.com>
  
     This file is part of libscanmem.
 
@@ -52,7 +53,7 @@
 # define EXPECT(x,y) x
 #endif
 
-// Dirty hack for FreeBSD
+// dirty hack for FreeBSD
 #if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 #define PTRACE_ATTACH PT_ATTACH
 #define PTRACE_DETACH PT_DETACH
@@ -61,12 +62,12 @@
 #endif
 
 #include "value.h"
-#include "endianness.h"
 #include "scanroutines.h"
 #include "scanmem.h"
 #include "show_message.h"
+#include "targetmem.h"
 
-/* Progress handling */
+/* progress handling */
 #define NUM_DOTS (10)
 #define NUM_SAMPLES (100)
 #define MAX_PROGRESS (1.0)  /* 100% */
@@ -76,14 +77,14 @@
 #define SAMPLES_PER_DOT (NUM_SAMPLES / NUM_DOTS)
 #define PROGRESS_PER_SAMPLE (MAX_PROGRESS / NUM_SAMPLES)
 
-/* ptrace peek buffer, used by peekdata() */
-/* make it larger in order to reduce shift */
-/* #define MAX_PEEKBUF_SIZE (4*sizeof(int64_t)) */
-#define MAX_PEEKBUF_SIZE (1024)
+/* ptrace peek buffer, used by peekdata() as a mirror of the process memory.
+ * Max size is the maximum allowed rounded VLT scan length, aka UINT16_MAX,
+ * plus a `long` for shifting efficiency */
+#define MAX_PEEKBUF_SIZE ((1<<16) + sizeof(long))
 static struct {
     uint8_t cache[MAX_PEEKBUF_SIZE];  /* read from ptrace()  */
     unsigned size;              /* number of entries (in bytes) */
-    char *base;                 /* base address of cached region */
+    const char *base;           /* base address of cached region */
     pid_t pid;                  /* what pid this applies to */
 } peekbuf;
 
@@ -92,7 +93,7 @@ bool sm_attach(pid_t target)
 {
     int status;
 
-    /* attach, to the target application, which should cause a SIGSTOP */
+    /* attach to the target application, which should cause a SIGSTOP */
     if (ptrace(PTRACE_ATTACH, target, NULL, NULL) == -1L) {
         show_error("failed to attach to %d, %s\n", target, strerror(errno));
         return false;
@@ -115,7 +116,7 @@ bool sm_attach(pid_t target)
 
 bool sm_detach(pid_t target)
 {
-    // addr is ignore under Linux, but should be 1 under FreeBSD in order to let the child process continue at what it had been interrupted
+    // addr is ignored on Linux, but should be 1 on FreeBSD in order to let the child process continue execution where it had been interrupted
     return ptrace(PTRACE_DETACH, target, 1, 0) == 0;
 }
 
@@ -123,86 +124,82 @@ bool sm_detach(pid_t target)
 /*
  * sm_peekdata - caches overlapping ptrace reads to improve performance.
  * 
- * This routine could just call ptrace(PEEKDATA), but using a cache reduces
- * the number of peeks required by 70% when reading large chunks of
- * consecutive addresses.
+ * This routine calls `ptrace(PEEKDATA, ...)`, and fills the peekbuf cache,
+ * to make a local mirror of the process memory we're interested in.
  */
 
-bool sm_peekdata(pid_t pid, void *addr, value_t * result)
+extern inline bool sm_peekdata(pid_t pid, const void *addr, uint16_t length, const mem64_t **result_ptr, size_t *memlength)
 {
-    char *reqaddr = addr;
+    const char *reqaddr = addr;
     int i, j;
-    int shift_size1 = 0, shift_size2 = 0;
-    char *last_address_gathered = 0;
+    unsigned int missing_bytes = 0;
 
     assert(peekbuf.size <= MAX_PEEKBUF_SIZE);
-    assert(result != NULL);
-
-    /* initialise result */
-    zero_value(result);
-    valnowidth(result);
+    assert(result_ptr != NULL);
+    assert(memlength != NULL);
 
     /* check if we have a cache hit */
     if (pid == peekbuf.pid &&
-            reqaddr >= peekbuf.base &&
-            (unsigned long) (reqaddr + sizeof(int64_t) - peekbuf.base) <= peekbuf.size) {
-        memcpy(&result->int64_value, &peekbuf.cache[reqaddr - peekbuf.base], sizeof(result->int64_value));
+        reqaddr >= peekbuf.base &&
+        (unsigned long) (reqaddr + length - peekbuf.base) <= peekbuf.size)
+    {
+        *result_ptr = (mem64_t*)&peekbuf.cache[reqaddr - peekbuf.base];
+        *memlength = peekbuf.base - reqaddr + peekbuf.size;
         return true;
-    } else if (pid == peekbuf.pid &&
-            reqaddr >= peekbuf.base &&
-            (unsigned long) (reqaddr - peekbuf.base) < peekbuf.size) {
+    }
+    else if (pid == peekbuf.pid &&
+             reqaddr >= peekbuf.base &&
+             (unsigned long) (reqaddr - peekbuf.base) < peekbuf.size)
+    {
 
         assert(peekbuf.size != 0);
 
         /* partial hit, we have some of the data but not all, so remove old entries - shift the frame by as far as is necessary */
-        /* tail shift, round up to nearest long size, for ptrace efficiency */
-        shift_size1 = (reqaddr + sizeof(int64_t)) - (peekbuf.base + peekbuf.size);
-        shift_size1 = sizeof(long) * (1 + (shift_size1-1) / sizeof(long));
+        /* missing bytes: round up to nearest long size for ptrace efficiency */
+        missing_bytes = (reqaddr + length) - (peekbuf.base + peekbuf.size);
+        missing_bytes = sizeof(long) * (1 + (missing_bytes-1) / sizeof(long));
 
-        /* head shift if necessary*/
-        if (peekbuf.size + shift_size1 > MAX_PEEKBUF_SIZE) 
+        /* head shift if necessary */
+        if (peekbuf.size + missing_bytes > MAX_PEEKBUF_SIZE)
         {
-            shift_size2 = reqaddr-peekbuf.base;
-            shift_size2 = sizeof(long) * (shift_size2 / sizeof(long));
+            int shift_size = reqaddr-peekbuf.base;
+            shift_size = sizeof(long) * (shift_size / sizeof(long));
 
-            for (i = shift_size2; i < peekbuf.size; ++i)
-            {
-                peekbuf.cache[i - shift_size2] = peekbuf.cache[i];
-            }
-            peekbuf.size -= shift_size2;
-            peekbuf.base += shift_size2;
+            memmove(peekbuf.cache, &peekbuf.cache[shift_size], peekbuf.size-shift_size);
+
+            peekbuf.size -= shift_size;
+            peekbuf.base += shift_size;
         }
-    } else {
-
-        /* cache miss, invalidate cache */
-        shift_size1 = shift_size2 = sizeof(int64_t);
+    }
+    else {
+        /* cache miss, invalidate the cache */
+        missing_bytes = length;
         peekbuf.pid = pid;
         peekbuf.size = 0;
         peekbuf.base = addr;
     }
-
-    /* we need a ptrace() to complete request */
+    /* we need a ptrace() to complete the request */
     errno = 0;
     
-    for (i = 0; i < shift_size1; i += sizeof(long))
+    for (i = 0; i < missing_bytes; i += sizeof(long))
     {
-        char *ptrace_address = peekbuf.base + peekbuf.size;
+        const char *ptrace_address = peekbuf.base + peekbuf.size;
         long ptraced_long = ptrace(PTRACE_PEEKDATA, pid, ptrace_address, NULL);
 
         /* check if ptrace() succeeded */
         if (EXPECT(ptraced_long == -1L && errno != 0, false)) {
-            /* its possible i'm trying to read partially oob */
+            /* it's possible i'm trying to read partially oob */
             if (errno == EIO || errno == EFAULT) {
                 
                 /* read backwards until we get a good read, then shift out the right value */
                 for (j = 1, errno = 0; j < sizeof(long); j++, errno = 0) {
                 
-                    /* Try for a shifted ptrace - 'continue' (i.e. try an increased shift) if it fails */
+                    /* try for a shifted ptrace - 'continue' (i.e. try an increased shift) if it fails */
                     if ((ptraced_long = ptrace(PTRACE_PEEKDATA, pid, ptrace_address - j, NULL)) == -1L && 
                         (errno == EIO || errno == EFAULT))
                             continue;
                     
-                    /* Cache it with the appropriate offset */
+                    /* cache it with the appropriate offset */
                     if(peekbuf.size >= j)
                     {
                         memcpy(&peekbuf.cache[peekbuf.size - j], &ptraced_long, sizeof(long));
@@ -213,54 +210,30 @@ bool sm_peekdata(pid_t pid, void *addr, value_t * result)
                         peekbuf.base -= j;
                     }
                     peekbuf.size += sizeof(long) - j;
-                    last_address_gathered = ptrace_address + sizeof(long) - j;
                     
-                    /* Interrupt the gathering process */
-                    goto doublebreak;
+                    /* interrupt the gathering process */
+                    break;
                 }
+
+                /* partial success */
+                goto end;
             }
             
-            /* i wont print a message here, would be very noisy if a region is unmapped */
+            /* I wont print a message here, would be very noisy if a region is unmapped */
+            *result_ptr = NULL;
+            *memlength = 0;
             return false;
         }
         
-        /* Otherwise, ptrace() worked - cache the data, increase the size */
+        /* otherwise, ptrace() worked - cache the data and increase the size */
         memcpy(&peekbuf.cache[peekbuf.size], &ptraced_long, sizeof(long));
         peekbuf.size += sizeof(long);
-        last_address_gathered = ptrace_address + sizeof(long);
-    }
-    
-    doublebreak:
-    
-    /* Return result to caller */
-    if (reqaddr + sizeof(int64_t) <= last_address_gathered)
-    {
-        /* The values are fine - read away */
-        memcpy(&result->int64_value, &peekbuf.cache[reqaddr - peekbuf.base], sizeof(result->int64_value));
-    }
-    else
-    {
-        int successful_gathering = last_address_gathered - reqaddr;
-        
-        /* We didn't get enough - add zeroes at the end */
-        for (i = 0; i < sizeof(int64_t); ++i)
-        {
-            uint8_t val = (i < successful_gathering) ? peekbuf.cache[reqaddr - peekbuf.base + i] : 0;
-            *(((uint8_t *)&result->int64_value)    + i) = val;
-        }
-        
-        /* Mark which values this can't be */
-        if (successful_gathering < sizeof(int64_t))
-            result->flags.u64b = result->flags.s64b = result->flags.f64b = 0;
-        if (successful_gathering < sizeof(int32_t))
-            result->flags.u32b = result->flags.s32b = result->flags.f32b = 0;
-        if (successful_gathering < sizeof(int16_t))
-            result->flags.u16b = result->flags.s16b = 0;
-        if (successful_gathering < sizeof(int8_t))
-            result->flags.u8b  = result->flags.s8b  = 0;
     }
 
-    /* success */
+end:
+    /* return result to caller */
+    *result_ptr = (mem64_t*)&peekbuf.cache[reqaddr - peekbuf.base];
+    *memlength = peekbuf.base - reqaddr + peekbuf.size;
     return true;
 }
 
@@ -270,23 +243,49 @@ static inline void print_a_dot(void)
     fflush(stderr);
 }
 
-/* This is the function that handles when you enter a value (or >, <, =) for the second or later time (i.e. when there's already a list of matches); it reduces the list to those that still match. It returns false on failure to attach, detach, or reallocate memory, otherwise true.
-"value" is what to compare to. It is meaningless when the match type is not MATCHEXACT. */
+static inline uint16_t flags_to_memlength(scan_data_type_t scan_data_type, match_flags flags)
+{
+    switch(scan_data_type)
+    {
+        case BYTEARRAY:
+        case STRING:
+            return flags;
+            break;
+        default: /* numbers */
+                 if (flags & flags_64b) return 8;
+            else if (flags & flags_32b) return 4;
+            else if (flags & flags_16b) return 2;
+            else if (flags & flags_8b ) return 1;
+            else    /* it can't be a variable of any size */ return 0;
+            break;
+    }
+}
+
+/* This is the function that handles when you enter a value (or >, <, =) for the second or later time (i.e. when there's already a list of matches);
+ * it reduces the list to those that still match. It returns false on failure to attach, detach, or reallocate memory, otherwise true. */
 bool sm_checkmatches(globals_t *vars,
                      scan_match_type_t match_type,
                      const uservalue_t *uservalue)
 {
-    matches_and_old_values_swath *reading_swath_index = (matches_and_old_values_swath *)vars->matches->swaths;
+    matches_and_old_values_swath *reading_swath_index = vars->matches->swaths;
     matches_and_old_values_swath reading_swath = *reading_swath_index;
 
-    long bytes_scanned = 0;
-    long total_scan_bytes = 0;
+    unsigned long bytes_scanned = 0;
+    unsigned long total_scan_bytes = 0;
     matches_and_old_values_swath *tmp_swath_index = reading_swath_index;
-    int samples_remaining = NUM_SAMPLES;
-    int samples_to_dot = SAMPLES_PER_DOT;
+    unsigned int samples_remaining = NUM_SAMPLES;
+    unsigned int samples_to_dot = SAMPLES_PER_DOT;
     size_t bytes_at_next_sample;
     size_t bytes_per_sample;
 
+    if (sm_choose_scanroutine(vars->options.scan_data_type, match_type, uservalue, vars->options.reverse_endianness) == false)
+    {
+        show_error("unsupported scan for current data type.\n");
+        return false;
+    }
+
+    assert(sm_scan_routine);
+
     while(tmp_swath_index->number_of_bytes)
     {
         total_scan_bytes += tmp_swath_index->number_of_bytes;
@@ -297,74 +296,67 @@ bool sm_checkmatches(globals_t *vars,
     /* for user, just print the first dot */
     print_a_dot();
 
-    int reading_iterator = 0;
-    matches_and_old_values_swath *writing_swath_index = (matches_and_old_values_swath *)vars->matches->swaths;
+    size_t reading_iterator = 0;
+    matches_and_old_values_swath *writing_swath_index = vars->matches->swaths;
     writing_swath_index->first_byte_in_child = NULL;
     writing_swath_index->number_of_bytes = 0;
-    
-    /* used to fill in non-match regions */
-    match_flags zero_flag;
-    zero_match_flags(&zero_flag);
 
     int required_extra_bytes_to_record = 0;
     vars->num_matches = 0;
     vars->scan_progress = 0.0;
-    
-    if (sm_choose_scanroutine(vars->options.scan_data_type, match_type) == false)
-    {
-        show_error("unsupported scan for current data type.\n"); 
-        return false;
-    }
+    vars->stop_flag = false;
 
-    assert(sm_scan_routine);
-    
     /* stop and attach to the target */
     if (sm_attach(vars->target) == false)
         return false;
+
     while (reading_swath.first_byte_in_child) {
-        int match_length = 0;
-        value_t data_value;
+        unsigned int match_length = 0;
+        const mem64_t *memory_ptr;
+        size_t memlength;
         match_flags checkflags;
 
+        match_flags old_flags = reading_swath_index->data[reading_iterator].match_info;
+        uint old_length = flags_to_memlength(vars->options.scan_data_type, old_flags);
         void *address = reading_swath.first_byte_in_child + reading_iterator;
-        
-        /* Read value from this address */
-        if (EXPECT(sm_peekdata(vars->target, address, &data_value) == false, false)) {
-            /* Uhh, what? We couldn't look at the data there? I guess this doesn't count as a match then */
-        }
-        else
+
+        /* read value from this address */
+        if (EXPECT(sm_peekdata(vars->target, address, old_length, &memory_ptr, &memlength) == false, false))
         {
-            value_t old_val = data_to_val_aux(reading_swath_index, reading_iterator, reading_swath.number_of_bytes /* ,MATCHES_AND_VALUES */);
-
-            match_flags flags = reading_swath_index->data[reading_iterator].match_info;
-            /* these are not harmful for bytearray routine, since it will ignore flags of new_value & old_value */
-            truncval_to_flags(&old_val, flags);
-            truncval_to_flags(&data_value, flags);
-
-            fix_endianness(vars, &data_value);
-
-            zero_match_flags(&checkflags);
-
-            match_length = (*sm_scan_routine)(&data_value, &old_val, uservalue, &checkflags, address);
+            /* If we can't look at the data here, just abort the whole recording, something bad happened */
+            required_extra_bytes_to_record = 0;
         }
-        
+        else if (old_flags != flags_empty) /* Test only valid old matches */
+        {
+            value_t old_val = data_to_val_aux(reading_swath_index, reading_iterator, reading_swath.number_of_bytes);
+            memlength = old_length < memlength ? old_length : memlength;
+
+            checkflags = flags_empty;
+
+            match_length = (*sm_scan_routine)(memory_ptr, memlength, &old_val, uservalue, &checkflags);
+        }
+
         if (match_length > 0)
         {
-            /* Still a candidate. Write data. 
-                (We can get away with overwriting in the same array because it is guaranteed to take up the same number of bytes or fewer, and because we copied out the reading swath metadata already.)
-                (We can get away with assuming that the pointers will stay valid, because as we never add more data to the array than there was before, it will not reallocate.) */
-          
-            old_value_and_match_info new_value = { get_u8b(&data_value), checkflags };
-            writing_swath_index = add_element((&vars->matches), writing_swath_index, address, &new_value /* ,MATCHES_AND_VALUES */);
-            
+            assert(match_length <= memlength);
+
+            /* Still a candidate. Write data.
+               - We can get away with overwriting in the same array because it is guaranteed to take up the same number of bytes or fewer,
+                 and because we copied out the reading swath metadata already.
+               - We can get away with assuming that the pointers will stay valid,
+                 because as we never add more data to the array than there was before, it will not reallocate. */
+
+            writing_swath_index = add_element(&(vars->matches), writing_swath_index, address,
+                                              get_u8b(memory_ptr), checkflags);
+
             ++vars->num_matches;
-            
+
             required_extra_bytes_to_record = match_length - 1;
         }
         else if (required_extra_bytes_to_record)
         {
-            old_value_and_match_info new_value = { get_u8b(&data_value), zero_flag };
-            writing_swath_index = add_element(&vars->matches, writing_swath_index, address, &new_value /* ,MATCHES_AND_VALUES */);
+            writing_swath_index = add_element(&(vars->matches), writing_swath_index, address,
+                                              get_u8b(memory_ptr), flags_empty);
             --required_extra_bytes_to_record;
         }
 
@@ -379,11 +371,13 @@ bool sm_checkmatches(globals_t *vars,
                     /* for user, just print a dot */
                     print_a_dot();
                 }
+                /* stop scanning if asked to */
+                if (vars->stop_flag) break;
             }
         }
         ++bytes_scanned;
         
-        /* Go on to the next one... */
+        /* go on to the next one... */
         ++reading_iterator;
         if (reading_iterator >= reading_swath.number_of_bytes)
         {
@@ -395,12 +389,14 @@ bool sm_checkmatches(globals_t *vars,
         }
     }
     
-    if (!(vars->matches = null_terminate(vars->matches, writing_swath_index /* ,MATCHES_AND_VALUES */)))
+    if (!(vars->matches = null_terminate(vars->matches, writing_swath_index)))
     {
         show_error("memory allocation error while reducing matches-array size\n");
         return false;
     }
 
+    show_user("ok\n");
+
     /* tell front-end we've done */
     vars->scan_progress = MAX_PROGRESS;
 
@@ -411,7 +407,7 @@ bool sm_checkmatches(globals_t *vars,
 }
 
 /* read region using /proc/pid/mem */
-ssize_t readregion(pid_t target, void *buf, size_t count, unsigned long offset)
+static inline ssize_t readregion(pid_t target, void *buf, size_t count, unsigned long offset)
 {
     char mem[32];
     int fd;
@@ -426,7 +422,7 @@ ssize_t readregion(pid_t target, void *buf, size_t count, unsigned long offset)
         return -1;
     }
 
-    /* try to honour the request */
+    /* try to honor the request */
     len = pread(fd, buf, count, offset);
     
     /* clean up */
@@ -436,7 +432,7 @@ ssize_t readregion(pid_t target, void *buf, size_t count, unsigned long offset)
 }
     
 
-/* sm_searchregions() performs an initial search of the process for values matching value */
+/* sm_searchregions() performs an initial search of the process for values matching `uservalue` */
 bool sm_searchregions(globals_t *vars, scan_match_type_t match_type, const uservalue_t *uservalue)
 {
     matches_and_old_values_swath *writing_swath_index;
@@ -446,18 +442,9 @@ bool sm_searchregions(globals_t *vars, scan_match_type_t match_type, const userv
     element_t *n = vars->regions->head;
     region_t *r;
     unsigned long total_scan_bytes = 0;
-    void *address = NULL;
-
-#if HAVE_PROCMEM
     unsigned char *data = NULL;
-    ssize_t len = 0;
-#endif
 
-    /* used to fill in non-match regions */
-    match_flags zero_flag;
-    zero_match_flags(&zero_flag);
-    
-    if (sm_choose_scanroutine(vars->options.scan_data_type, match_type) == false)
+    if (sm_choose_scanroutine(vars->options.scan_data_type, match_type, uservalue, vars->options.reverse_endianness) == false)
     {
         show_error("unsupported scan for current data type.\n"); 
         return false;
@@ -494,21 +481,22 @@ bool sm_searchregions(globals_t *vars, scan_match_type_t match_type, const userv
         return false;
     }
     
-    writing_swath_index = (matches_and_old_values_swath *)vars->matches->swaths;
+    writing_swath_index = vars->matches->swaths;
     
     writing_swath_index->first_byte_in_child = NULL;
     writing_swath_index->number_of_bytes = 0;
     
-    /* get total byte */
+    /* get total number of bytes */
     for(n = vars->regions->head; n; n = n->next)
         total_scan_bytes += ((region_t *)n->data)->size;
 
     vars->scan_progress = 0.0;
+    vars->stop_flag = false;
     n = vars->regions->head;
 
     /* check every memory region */
     while (n) {
-        unsigned offset, nread = 0;
+        size_t nread = 0;
         int dots_remaining = NUM_DOTS;
         size_t bytes_at_next_dot;
         size_t bytes_per_dot;
@@ -520,13 +508,20 @@ bool sm_searchregions(globals_t *vars, scan_match_type_t match_type, const userv
         bytes_at_next_dot = bytes_per_dot;
         progress_per_dot = (double)bytes_per_dot / total_scan_bytes;
 
-#if HAVE_PROCMEM        
-        /* over allocate by enough bytes set to zero that the last bytes can be read as 64-bit ints */
-        if ((data = calloc(r->size + sizeof(int64_t) - 1, 1)) == NULL) {
+        /* allocate data array */
+        if ((data = malloc(r->size * sizeof(char))) == NULL) {
             show_error("sorry, there was a memory allocation error.\n");
             return false;
         }
-    
+
+        /* print a progress meter so user knows we haven't crashed */
+        /* cannot use show_info here because it'll append a '\n' */
+        show_user("%02u/%02u searching %#10lx - %#10lx", ++regnum,
+                vars->regions->size, (unsigned long)r->start, (unsigned long)r->start + r->size);
+        fflush(stderr);
+
+#if HAVE_PROCMEM
+        ssize_t len = 0;
         /* keep reading until completed */
         while (nread < r->size) {
             if ((len = readregion(vars->target, data+nread, r->size-nread, (unsigned long)(r->start+nread))) == -1) {
@@ -538,72 +533,44 @@ bool sm_searchregions(globals_t *vars, scan_match_type_t match_type, const userv
             }
         }
 
-#else     
-        /* cannot use /proc/pid/mem */
-        nread = r->size;
-#endif
-        /* print a progress meter so user knows we havent crashed */
-        /* cannot use show_info here because it'll append a '\n' */
-        show_user("%02u/%02u searching %#10lx - %#10lx.", ++regnum,
-                vars->regions->size, (unsigned long)r->start, (unsigned long)r->start + r->size);
-        fflush(stderr);
-
-        /* for every offset, check if we have a match */
-        for (offset = 0; offset < nread; offset++) {
-            match_flags checkflags;
-            value_t data_value;
-           
-            /* initialise data_value */
-            zero_value(&data_value);
-            valnowidth(&data_value);
-
-            address = r->start + offset;
-
-#if HAVE_PROCMEM
-            /* Don't dereference as this causes an alignment issue e.g. on ARM.
-               GCC replaces memcpy() with dereferencing where possible. */
-            memcpy(&data_value.int64_value, &data[offset], sizeof(int64_t));
-
-            /* Mark which values this can't be */
-            if (EXPECT((nread - offset < sizeof(int64_t)), false))
-            {
-                data_value.flags.u64b = data_value.flags.s64b = data_value.flags.f64b = 0;
-                if (nread - offset < sizeof(int32_t))
-                {
-                    data_value.flags.u32b = data_value.flags.s32b = data_value.flags.f32b = 0;
-                    if (nread - offset < sizeof(int16_t))
-                    {
-                        data_value.flags.u16b = data_value.flags.s16b = 0;
-                        if (nread - offset < sizeof(int8_t))
-                        {
-                            data_value.flags.u8b  = data_value.flags.s8b  = 0;
-                        }
-                    }
-                }
-            }
 #else
-            if (EXPECT(peekdata(vars->target, address, &data_value) == false, false)) {
+        /* Read the region with `ptrace()`: the API specifies that `ptrace()` returns a `long`, which
+         * is the size of a word for the current architecture, so this section will deal in `long`s */
+        for (nread = 0; nread < r->size; nread += sizeof(long)) {
+            const char *ptrace_address = r->start + nread;
+            long ptraced_long = ptrace(PTRACE_PEEKDATA, vars->target, ptrace_address, NULL);
+
+            /* check if ptrace() succeeded */
+            if (EXPECT(ptraced_long == -1L && errno != 0, false)) {
+                /* interrupt the gathering process */
                 break;
             }
+
+            /* otherwise, ptrace() worked - store the data */
+            memcpy(data+nread, &ptraced_long, sizeof(long));
+        }
 #endif
+        /* region has been read, tell user */
+        print_a_dot();
 
-            fix_endianness(vars, &data_value);
+        /* For every offset, check if we have a match.
+         * Testing `memlength > 0` is much faster than `offset < nread` */
+        size_t memlength, offset;
+        for (memlength = nread, offset = 0; memlength > 0; memlength--, offset++) {
+            unsigned int match_length;
+            const mem64_t* memory_ptr = (mem64_t*)(data+offset);
+            match_flags checkflags;
 
-            /* initialise checkflags */
-            zero_match_flags(&checkflags);
+            /* initialize checkflags */
+            checkflags = flags_empty;
 
-            int match_length;
             /* check if we have a match */
-            if (EXPECT(((match_length = (*sm_scan_routine)(&data_value, NULL,
-                    uservalue, &checkflags, address)) > 0), false)) {
-                /* only set these flags for numbers */
-                if ((vars->options.scan_data_type != BYTEARRAY)
-                    && (vars->options.scan_data_type != STRING))
-                {
-                    checkflags.ineq_forwards = checkflags.ineq_reverse = 1;
-                }
-                old_value_and_match_info new_value = { get_u8b(&data_value), checkflags };
-                writing_swath_index = add_element((&vars->matches), writing_swath_index, r->start + offset, &new_value);
+            match_length = (*sm_scan_routine)(memory_ptr, memlength, NULL, uservalue, &checkflags);
+            if (EXPECT(match_length > 0, false))
+            {
+                assert(match_length <= memlength);
+                writing_swath_index = add_element(&(vars->matches), writing_swath_index, r->start+offset,
+                                                  get_u8b(memory_ptr), checkflags);
                 
                 ++vars->num_matches;
                 
@@ -611,12 +578,12 @@ bool sm_searchregions(globals_t *vars, scan_match_type_t match_type, const userv
             }
             else if (required_extra_bytes_to_record)
             {
-                old_value_and_match_info new_value = { get_u8b(&data_value), zero_flag };
-                writing_swath_index = add_element((&vars->matches), writing_swath_index, r->start + offset, &new_value);
+                writing_swath_index = add_element(&(vars->matches), writing_swath_index, r->start+offset,
+                                                  get_u8b(memory_ptr), flags_empty);
                 --required_extra_bytes_to_record;
             }
 
-            /* print a simple progress meter. */
+            /* print a simple progress meter */
             if (EXPECT(offset >= bytes_at_next_dot, false)) {
                 bytes_at_next_dot += bytes_per_dot;
                 /* handle rounding */
@@ -625,22 +592,24 @@ bool sm_searchregions(globals_t *vars, scan_match_type_t match_type, const userv
                     print_a_dot();
                     /* for front-end, update percentage */
                     vars->scan_progress += progress_per_dot;
+                    /* stop scanning if asked to */
+                    if (vars->stop_flag) break;
                 }
             }
         }
 
+        free(data);
         vars->scan_progress += progress_per_dot;
+        /* stop scanning if asked to */
+        if (vars->stop_flag) break;
         n = n->next;
         show_user("ok\n");
-#if HAVE_PROCMEM
-        free(data);
-#endif
     }
 
-    /* tell front-end we've done */
+    /* tell front-end we've finished */
     vars->scan_progress = MAX_PROGRESS;
     
-    if (!(vars->matches = null_terminate(vars->matches, writing_swath_index /* ,MATCHES_AND_VALUES */)))
+    if (!(vars->matches = null_terminate(vars->matches, writing_swath_index)))
     {
         show_error("memory allocation error while reducing matches-array size\n");
         return false;
@@ -652,42 +621,39 @@ bool sm_searchregions(globals_t *vars, scan_match_type_t match_type, const userv
     return sm_detach(vars->target);
 }
 
+/* Needs to support only ANYNUMBER types */
 bool sm_setaddr(pid_t target, void *addr, const value_t *to)
 {
-    value_t saved;
-    int i;
+    unsigned int i;
+    const mem64_t *memory_ptr;
+    size_t memlength;
 
     if (sm_attach(target) == false) {
         return false;
     }
 
-    if (sm_peekdata(target, addr, &saved) == false) {
-        show_error("couldnt access the target address %10p\n", addr);
+    if (sm_peekdata(target, addr, sizeof(uint64_t), &memory_ptr, &memlength) == false) {
+        show_error("couldn't access the target address %10p\n", addr);
         return false;
     }
-    
-    /* Basically, overwrite as much of the data as makes sense, and no more. */
-    /* about float/double: now value_t is a union, we can use the following way instead of the commented way, in order to avoid compiler warning */
-         if (saved.flags.u64b && to->flags.u64b) { set_u64b(&saved, get_u64b(to)); }
-    else if (saved.flags.s64b && to->flags.s64b) { set_s64b(&saved, get_s64b(to)); }
-    else if (saved.flags.f64b && to->flags.f64b) { set_s64b(&saved, get_s64b(to)); } /* *((int64_t *)&(to->float64_value))); } */
-    else if (saved.flags.u32b && to->flags.u32b) { set_u32b(&saved, get_u32b(to)); }
-    else if (saved.flags.s32b && to->flags.s32b) { set_s32b(&saved, get_s32b(to)); }
-    else if (saved.flags.f32b && to->flags.f32b) { set_s32b(&saved, get_s32b(to)); } /* *((int32_t *)&(to->float32_value))); } */
-    else if (saved.flags.u16b && to->flags.u16b) { set_u16b(&saved, get_u16b(to)); }
-    else if (saved.flags.s16b && to->flags.s16b) { set_s16b(&saved, get_s16b(to)); }
-    else if (saved.flags.u8b  && to->flags.u8b ) { set_u8b(&saved, get_u8b(to)); }
-    else if (saved.flags.s8b  && to->flags.s8b ) { set_s8b(&saved, get_s8b(to)); }
+
+    /* Assume `sizeof(uint64_t)` is a multiple of `sizeof(long)` */
+    long memarray[sizeof(uint64_t)/sizeof(long)] = {0};
+    uint val_length = flags_to_memlength(ANYNUMBER, to->flags);
+    if (val_length > 0) {
+        /* Basically, overwrite as much of the data as makes sense, and no more. */
+        memcpy(memarray, memory_ptr, memlength);
+        memcpy(memarray, to->bytes, val_length);
+    }
     else {
         show_error("could not determine type to poke.\n");
         return false;
     }
 
     /* TODO: may use /proc/<pid>/mem here */
-    /* assume that sizeof(save.int64_value) (int64_t) is multiple of sizeof(long) */
-    for (i = 0; i < sizeof(saved.int64_value); i += sizeof(long)) 
+    for (i = 0; i < sizeof(uint64_t)/sizeof(long); i++)
     {
-        if (ptrace(PTRACE_POKEDATA, target, addr + i, *(long *)(((int8_t *)&saved.int64_value) + i)) == -1L) {
+        if (ptrace(PTRACE_POKEDATA, target, addr + i*sizeof(long), memarray[i]) == -1L) {
             return false;
         }
     }
@@ -695,7 +661,7 @@ bool sm_setaddr(pid_t target, void *addr, const value_t *to)
     return sm_detach(target);
 }
 
-bool sm_read_array(pid_t target, void *addr, char *buf, int len)
+bool sm_read_array(pid_t target, const void *addr, char *buf, int len)
 {
     if (sm_attach(target) == false) {
         return false;
@@ -765,7 +731,7 @@ bool sm_write_array(pid_t target, void *addr, const void *data, int len)
         }
         else /* we have to play with bits... */
         {
-            /* try all possible shifting read & write */
+            /* try all possible shifting read and write */
             for(j = 0; j <= sizeof(long) - (len - i); ++j)
             {
                 errno = 0;
