@@ -25,54 +25,26 @@
 #ifndef SCANMEM_H
 #define SCANMEM_H
 
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdint.h>
-#include <string.h>
 #include <sys/types.h>
 
 #include "scanroutines.h"
 #include "list.h"
+#include "maps.h"
 #include "value.h"
 #include "targetmem.h"
 
 
-#ifndef PACKAGE_VERSION
-#define PACKAGE_VERSION "(unknown)"
-#endif
-
-/* from string.h in glibc for Android/BSD */
-#ifndef strdupa
-#define strdupa(s)                                                            \
-    ({                                                                        \
-      const char *__old = (s);                                                \
-      size_t __len = strlen(__old) + 1;                                       \
-      char *__new = (char *) alloca(__len);                                   \
-      (char *) memcpy(__new, __old, __len);                                   \
-    })
-#endif
-
-#ifndef strndupa
-#define strndupa(s, n)                                                        \
-    ({                                                                        \
-      const char *__old = (s);                                                \
-      size_t __len = strnlen(__old, (n));                                     \
-      char *__new = (char *) alloca(__len + 1);                               \
-      __new[__len] = '\0';                                                    \
-      (char *) memcpy(__new, __old, __len);                                   \
-    })
-#endif
-
-#ifndef MIN
-#define MIN(a,b) ((a) < (b) ? (a) : (b))
-#endif
-
 /* global settings */
 typedef struct {
     unsigned exit:1;
     pid_t target;
     matches_and_old_values_array *matches;
-    long num_matches;
+    unsigned long num_matches;
     double scan_progress;
+    bool stop_flag;
     list_t *regions;
     list_t *commands;              /* command handlers */
     const char *current_cmdline;   /* the command being executed */
@@ -81,12 +53,11 @@ typedef struct {
         unsigned short alignment;
         unsigned short debug;
         unsigned short backend;    /* if 1, scanmem will work as a backend and
-                                      output would be more machine-readable */
+                                      output will be more machine-readable */
 
         /* options that can be changed during runtime */
         scan_data_type_t scan_data_type;
         region_scan_level_t region_scan_level;
-        unsigned short detect_reverse_change;
         unsigned short dump_with_ascii;
         unsigned short reverse_endianness;
     } options;
@@ -96,13 +67,15 @@ typedef struct {
 extern globals_t sm_globals;
 
 bool sm_init(void);
+void sm_cleanup(void);
 void sm_printversion(FILE *outfd);
 void sm_set_backend(void);
 void sm_backend_exec_cmd(const char *commandline);
-long sm_get_num_matches(void);
+unsigned long sm_get_num_matches(void);
 const char *sm_get_version(void);
 double sm_get_scan_progress(void);
 void sm_reset_scan_progress(void);
+void sm_set_stop_flag(bool stop_flag);
 
 /* ptrace.c */
 bool sm_detach(pid_t target);
@@ -111,12 +84,9 @@ bool sm_checkmatches(globals_t *vars, scan_match_type_t match_type,
                      const uservalue_t *uservalue);
 bool sm_searchregions(globals_t *vars, scan_match_type_t match_type,
                       const uservalue_t *uservalue);
-bool sm_peekdata(pid_t pid, void *addr, value_t *result);
+bool sm_peekdata(pid_t pid, const void *addr, uint16_t length, const mem64_t **result_ptr, size_t *memlength);
 bool sm_attach(pid_t target);
-bool sm_read_array(pid_t target, void *addr, char *buf, int len);
+bool sm_read_array(pid_t target, const void *addr, char *buf, int len);
 bool sm_write_array(pid_t target, void *addr, const void *data, int len);
 
-/* menu.c */
-bool sm_getcommand(globals_t *vars, char **line);
-
 #endif /* SCANMEM_H */
